<?php
/**
 * @file
 * Handle the 'section view' override task.
 *
 * This plugin overrides section/%section and reroutes it to the page manager,
 * where a list of tasks can be used to service this request based upon criteria
 * supplied by access plugins.
 */

/**
 * Specialized implementation of hook_page_manager_task_tasks(). See
 * api-task.html for more information.
 */
function section_section_view_page_manager_tasks() {
  return array(
    // This is a 'page' task and will fall under the page admin UI.
    'task type' => 'page',

    'title' => t('Section template'),

    'admin title' => t('Section template'),
    'admin description' => t('When enabled, this overrides the default behavior for displaying sections at <em>section/%section</em>.'),
    'admin path' => 'section/%section',

    // Menu hooks so that we can alter the section/%section menu entry to
    // point to us.
    'hook menu' => 'section_section_view_menu',
    'hook menu alter' => 'section_section_view_menu_alter',

    // This is task uses 'context' handlers and must implement these to give the
    // handler data it needs.
    'handler type' => 'context',
    'get arguments' => 'section_section_view_get_arguments',
    'get context placeholders' => 'section_section_view_get_contexts',

    // Allow this to be enabled or disabled:
    'disabled' => variable_get('section_section_view_disabled', TRUE),
    'enable callback' => 'section_section_view_enable',
  );
}

/**
 * Callback defined by section_section_view_page_manager_tasks().
 *
 * Alter the node view input so that node view comes to us rather than the
 * normal node view process.
 */
function section_section_view_menu_alter(&$items, $task) {
  if (variable_get('section_section_view_disabled', TRUE)) {
    return;
  }

  // Override the node view handler for our purpose.
  $callback = $items['section/%section']['page callback'];
  if ($callback === 'section_page_view' || variable_get('page_manager_override_anyway', FALSE)) {
    $items['section/%section']['page callback'] = 'section_section_view_page';
    $items['section/%section']['file path'] = $task['path'];
    $items['section/%section']['file'] = $task['file'];
  }
  else {
    // Automatically disable this task if it cannot be enabled.
    variable_set('section_section_view_disabled', TRUE);
    if (!empty($GLOBALS['section_enabling_section_view'])) {
      drupal_set_message(t('Page manager module is unable to enable section/%section because some other module already has overridden with %callback.', array('%callback' => $callback)), 'error');
    }
  }
}

/**
 * Entry point for our overridden section view.
 *
 * This function asks its assigned handlers who, if anyone, would like
 * to run with it. If no one does, it passes through to sections
 * section view, which is section_page_view().
 */
function section_section_view_page($section) {
  // Load my task plugin.
  $task = page_manager_get_task('section_view');

  // Load the node into a context.
  ctools_include('context');
  ctools_include('context-task-handler');
  $contexts = ctools_context_handler_get_task_contexts($task, '', array($section));

  $output = ctools_context_handler_render($task, '', $contexts, array($section->sid));
  if ($output != FALSE) {
    return $output;
  }

  $function = 'section_page_view';
  foreach (module_implements('page_manager_override') as $module) {
    $call = $module . '_page_manager_override';
    if (($rc = $call('section_view')) && function_exists($rc)) {
      $function = $rc;
      break;
    }
  }

  // Otherwise, fall back.
  return $function($section);
}

/**
 * Callback to get arguments provided by this task handler.
 *
 * Since this is the section view and there is no UI on the arguments, we
 * create dummy arguments that contain the needed data.
 */
function section_section_view_get_arguments($task, $subtask_id) {
  return array(
    array(
      'keyword' => 'section',
      'identifier' => t('Section being viewed'),
      'id' => 1,
      'name' => 'entity_id:section',
      'settings' => array(),
    ),
  );
}

/**
 * Callback to get context placeholders provided by this handler.
 */
function section_section_view_get_contexts($task, $subtask_id) {
  return ctools_context_get_placeholders_from_argument(section_section_view_get_arguments($task, $subtask_id));
}

/**
 * Callback to enable/disable the page from the UI.
 */
function section_section_view_enable($cache, $status) {
  variable_set('section_section_view_disabled', $status);

  // Set a global flag so that the menu routine knows it needs
  // to set a message if enabling cannot be done.
  if (!$status) {
    $GLOBALS['section_enabling_section_view'] = TRUE;
  }
}
