<?php
/**
 * @file
 * Integration with menu module.
 */

/**
 * Save section menu item.
 */
function section_save_menu_item($section) {
  if (isset($section->menu)) {
    $link = &$section->menu;
    if (empty($link['enabled'])) {
      if (!empty($link['mlid'])) {
        menu_link_delete($link['mlid']);
      }
    }
    elseif (trim($link['link_title'])) {
      // Get section uri.
      $uri = section_uri($section);

      $link['link_title'] = trim($link['link_title']);
      $link['link_path'] = $uri['path'];

      if (trim($link['description'])) {
        $link['options']['attributes']['title'] = trim($link['description']);
      }
      else {
        // If the description field was left empty, remove the title attribute
        // from the menu link.
        unset($link['options']['attributes']['title']);
      }

      if (!menu_link_save($link)) {
        drupal_set_message(t('There was an error saving the menu link.'), 'error');
      }
    }
  }
}

/**
 * Alter section_form.
 */
function _section_menu_section_form(&$form, $form_state) {
  $bundle = section_bundle_load($form_state['section']->bundle);
  $bundle_settings = unserialize($bundle->settings);

  $link = $form_state['section']->menu;
  $bundle = $form_state['section']->bundle;

  // menu_parent_options() is goofy and can actually handle either a menu link
  // or a node type both as second argument. Pick based on whether there is
  // a link already (menu_node_prepare() sets mlid default to 0).
  $options = menu_parent_options(menu_get_menus(), $link['mlid'] ? $link : $bundle);

  // If no possible parent menu items were found, there is nothing to display.
  if (empty($options)) {
    return;
  }

  $form['menu'] = array(
    '#type' => 'fieldset',
    '#title' => t('Menu settings'),
    '#access' => user_access('administer menu'),
    '#collapsible' => TRUE,
    '#collapsed' => !$link['link_title'],
    '#group' => 'additional_settings',
    '#attached' => array(
      'js' => array(drupal_get_path('module', 'section') . '/section.menu.js'),
    ),
    '#tree' => TRUE,
    '#weight' => 11,
    '#attributes' => array('class' => array('menu-link-form')),
  );
  $form['menu']['enabled'] = array(
    '#type' => 'checkbox',
    '#title' => t('Provide a menu link'),
    '#default_value' => (int) (bool) $link['mlid'],
  );
  $form['menu']['link'] = array(
    '#type' => 'container',
    '#parents' => array('menu'),
    '#states' => array(
      'invisible' => array(
        'input[name="menu[enabled]"]' => array('checked' => FALSE),
      ),
    ),
  );

  // Populate the element with the link data.
  $elements = array(
    'mlid',
    'module',
    'hidden',
    'has_children',
    'customized',
    'options',
    'expanded',
    'hidden',
    'parent_depth_limit',
  );
  foreach ($elements as $key) {
    $form['menu']['link'][$key] = array('#type' => 'value', '#value' => $link[$key]);
  }

  $form['menu']['link']['link_title'] = array(
    '#type' => 'textfield',
    '#title' => t('Menu link title'),
    '#default_value' => $link['link_title'],
  );

  $form['menu']['link']['description'] = array(
    '#type' => 'textarea',
    '#title' => t('Description'),
    '#default_value' => isset($link['options']['attributes']['title']) ? $link['options']['attributes']['title'] : '',
    '#rows' => 1,
    '#description' => t('Shown when hovering over the menu link.'),
  );

  $default = ($link['mlid'] ? $link['menu_name'] . ':' . $link['plid'] : isset($bundle_settings['menu_parent']) ? $bundle_settings['menu_parent'] : 'main-menu:0');

  // If the current parent menu item is not present in options, use the first
  // available option as default value.
  // @todo User should not be allowed to access menu link settings in such a
  // case.
  if (!isset($options[$default])) {
    $array = array_keys($options);
    $default = reset($array);
  }
  $form['menu']['link']['parent'] = array(
    '#type' => 'select',
    '#title' => t('Parent item'),
    '#default_value' => $default,
    '#options' => $options,
    '#attributes' => array('class' => array('menu-parent-select')),
  );
  $form['menu']['link']['weight'] = array(
    '#type' => 'weight',
    '#title' => t('Weight'),
    '#delta' => 50,
    '#default_value' => $link['weight'],
    '#description' => t('Menu links with smaller weights are displayed before links with larger weights.'),
  );
}

/**
 * Form submit handler.
 */
function _section_menu_section_form_submit($form, &$form_state) {
  // Decompose the selected menu parent option into 'menu_name' and 'plid', if
  // the form used the default parent selection widget.
  if (!empty($form_state['values']['menu']['parent'])) {
    list($form_state['section']->menu['menu_name'], $form_state['section']->menu['plid']) = explode(':', $form_state['values']['menu']['parent']);
  }

  // Save the menu item.
  section_save_menu_item($form_state['section']);
  menu_rebuild();
}

/**
 * Alter section prepare.
 */
function _section_menu_section_prepare(&$section) {
  $bundle = section_bundle_load($section->bundle);
  $bundle_settings = unserialize($bundle->settings);

  if (empty($section->menu)) {
    // Prepare the node for the edit form so that $node->menu always exists.
    $menu_name = isset($bundle_settings['menu_parent']) ? $bundle_settings['menu_parent'] : 'main-menu:0';
    $menu_name = strtok($menu_name, ':');

    $item = array();
    if (isset($section->name)) {
      // Get section uri.
      $uri = section_uri($section);

      $mlid = FALSE;
      // Give priority to the default menu.
      $type_menus = isset($bundle_settings['menu_options']) ? $bundle_settings['menu_options'] : array('main-menu' => 'main-menu');
      if (in_array($menu_name, $type_menus)) {
        $mlid = db_query_range("SELECT mlid FROM {menu_links} WHERE link_path = :path AND menu_name = :menu_name AND module = '´section' ORDER BY mlid ASC", 0, 1, array(
          ':path' => $uri['path'],
          ':menu_name' => $menu_name,
        ))->fetchField();
      }
      // Check all allowed menus if a link does not exist in the default menu.
      if (!$mlid && !empty($type_menus)) {
        $mlid = db_query_range("SELECT mlid FROM {menu_links} WHERE link_path = :path AND module = 'section' AND menu_name IN (:type_menus) ORDER BY mlid ASC", 0, 1, array(
          ':path' => $uri['path'],
          ':type_menus' => array_values($type_menus),
        ))->fetchField();
      }
      if ($mlid) {
        $item = menu_link_load($mlid);
      }
    }
    // Set default values.
    $section->menu = $item + array(
      'link_title' => '',
      'mlid' => 0,
      'plid' => 0,
      'menu_name' => $menu_name,
      'weight' => 0,
      'options' => array(),
      'module' => 'section',
      'expanded' => 0,
      'hidden' => 0,
      'has_children' => 0,
      'customized' => 0,
    );
  }
  // Find the depth limit for the parent select.
  if (!isset($section->menu['parent_depth_limit'])) {
    $section->menu['parent_depth_limit'] = _menu_parent_depth_limit($section->menu);
  }
}
