<?php
/**
 * @file
 * Section pages.
 */

/**
 * Overview page for creating sections.
 */
function section_add_page() {
  $item = menu_get_item();
  $bundles = system_admin_menu_block($item);

  // Bypass the section/add listing if only one section bundle is available.
  if (count($bundles) == 1) {
    $item = array_shift($bundles);
    drupal_goto($item['href']);
  }
  return theme('section_add_list', array('bundles' => $bundles));
}

/**
 * Returns HTML for a list of available asection bundles for section creation.
 */
function theme_section_add_list($variables) {
  $bundles = $variables['bundles'];
  $output = '';

  if ($bundles) {
    $output = '<dl class="node-type-list">';
    foreach ($bundles as $item) {
      $output .= '<dt>' . l($item['title'], $item['href'], $item['localized_options']) . '</dt>';
      $output .= '<dd>' . filter_xss_admin($item['description']) . '</dd>';
    }
    $output .= '</dl>';
  }
  else {
    $output = '<p>' . t('You have not created any sections types yet. Go to the <a href="@create-sections">section type creation page</a> to add a new section type.', array('@create-section' => url('admin/structure/section_bundle/add'))) . '</p>';
  }
  return $output;
}

/**
 * Returns a section form.
 */
function section_add($bundle) {
  global $user;

  $bundles = section_bundles();
  $section = entity_create('section', array('bundle' => $bundle));
  drupal_set_title(t('Create @name', array('@name' => $bundles[$bundle]->name)), PASS_THROUGH);
  $output = drupal_get_form($bundle . '_section_form', $section);

  return $output;
}

/**
 * Menu callback; presents the section editing form.
 */
function section_page_edit($section) {
  drupal_set_title(t('<em>Edit</em> @title', array('@title' => $section->label)), PASS_THROUGH);
  return drupal_get_form($section->bundle . '_section_form', $section);
}

/**
 * Generate the section add/edit form array.
 */
function section_form($form, &$form_state, $section) {
  // During initial form build, add the section entity to the form state for use
  // during form building and processing. During a rebuild, use what is in the
  // form state.
  if (!isset($form_state['section'])) {
    if (!isset($section->label)) {
      $section->label = NULL;
    }
    if (!isset($section->name)) {
      $section->name = NULL;
    }
    section_object_prepare($section);
    $form_state['section'] = $section;
  }
  else {
    $section = $form_state['section'];
  }

  $form['label'] = array(
    '#type' => 'textfield',
    '#title' => t('Label'),
    '#default_value' => $section->label,
    '#size' => 60,
    '#maxlength' => 128,
    '#required' => TRUE,
    '#description' => t('This label is just for the readable machine name.'),
    '#weight' => -10,
  );

  $form['name'] = array(
    '#type' => 'machine_name',
    '#default_value' => $section->name,
    '#maxlength' => 128,
    '#machine_name' => array(
      'exists' => 'section_name_exists',
      'source' => array('label'),
    ),
    '#weight' => -9,
  );

  // Section options for administrators.
  $form['options'] = array(
    '#type' => 'fieldset',
    '#access' => user_access('administer sections'),
    '#title' => t('Publishing options'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#group' => 'additional_settings',
    '#attributes' => array(
      'class' => array('section-form-options'),
    ),
    '#attached' => array(
      'js' => array(drupal_get_path('module', 'section') . '/section.js'),
    ),
    '#weight' => 95,
  );
  $form['options']['published'] = array(
    '#type' => 'checkbox',
    '#title' => t('Published'),
    '#default_value' => $section->published,
  );

  $form['additional_settings'] = array(
    '#type' => 'vertical_tabs',
    '#weight' => 99,
  );

  // Add the buttons.
  $form['actions'] = array('#type' => 'actions');
  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
    '#weight' => 5,
    '#submit' => array('section_form_submit'),
  );

  if (!empty($section->sid) && section_access('delete', $section)) {
    $form['actions']['delete'] = array(
      '#type' => 'submit',
      '#value' => t('Delete'),
      '#weight' => 15,
      '#submit' => array('section_form_delete_submit'),
    );
  }

  $form['#validate'][] = 'section_form_validate';
  if (!isset($form['#submit']) && function_exists($section->bundle . '_section_form_submit')) {
    $form['#submit'][] = $section->bundle . '_section_form_submit';
  }
  $form += array('#submit' => array());

  field_attach_form('section', $section, $form, $form_state);

  // Integration with menu.
  if (module_exists('menu')) {
    _section_menu_section_prepare($form_state['section']);
    _section_menu_section_form($form, $form_state);
  }

  // Integration with path.
  if (module_exists('path')) {
    _section_path_section_form($form, $form_state);
  }

  // Integration with pathauto.
  if (module_exists('pathauto')) {
    pathauto_field_attach_form('section', $section, $form, $form_state, LANGUAGE_NONE);
  }

  return $form;
}

/**
 * Validate callback; From section_form.
 */
function section_form_validate($form, &$form_state) {
}

/**
 * Submit callback; From section_form.
 */
function section_form_submit($form, &$form_state) {
  $section = $form_state['section'];

  // Build a section entity based on the $form_state values.
  entity_form_submit_build_entity('section', $section, $form, $form_state);

  // Save the section.
  $status = entity_save('section', $section);

  $watchdog_args = array('@bundle' => $section->bundle, '%title' => $section->label);
  $t_args = array('@bundle' => section_bundle_get_name($section), '%title' => $section->label);
  $section_link = l(t('view'), 'section/' . $section->sid);

  if ($status == SAVED_NEW) {
    watchdog('section', '@bundle: added section %title.', $watchdog_args, WATCHDOG_NOTICE, $section_link);
    drupal_set_message(t('@bundle: %title has been created.', $t_args));
  }
  elseif ($status == SAVED_UPDATED) {
    watchdog('section', '@bundle: updated section %title.', $watchdog_args, WATCHDOG_NOTICE, $section_link);
    drupal_set_message(t('@bundle: %title has been updated.', $t_args));
  }

  if ($section->sid) {
    $form_state['values']['name'] = $section->sid;
    $form_state['name'] = $section->sid;
    $form_state['redirect'] = 'section/' . $section->sid;
  }
  else {
    // In the unlikely case something went wrong on save, the section will be
    // rebuilt and node form redisplayed the same way as in preview.
    drupal_set_message(t('The post could not be saved.'), 'error');
    $form_state['rebuild'] = TRUE;
  }

  // Integration with menu.
  if (module_exists('menu')) {
    _section_menu_section_form_submit($form, $form_state);
  }
}

/**
 * Returns whether a section name already exists.
 *
 * @return bool
 *   TRUE if section exists, FALSE if not.
 */
function section_name_exists($value) {
  $result = db_query("SELECT name FROM {section} WHERE name = :value", array(':value' => $value));
  $exists = $result->rowCount();
  return (bool) $exists;
}


/**
 * Button submit function: handle the 'Delete' button on the section form.
 */
function section_form_delete_submit($form, &$form_state) {
  $destination = array();
  if (isset($_GET['destination'])) {
    $destination = drupal_get_destination();
    unset($_GET['destination']);
  }
  $section = $form_state['section'];
  $form_state['redirect'] = array('section/' . $section->sid . '/delete', array('query' => $destination));
}

/**
 * Menu callback -- ask for confirmation of section deletion
 */
function section_delete_confirm($form, &$form_state, $section) {
  // @todo shouldn't we use form_state here?
  $form['#section'] = $section;

  // Always provide entity id in the same form key as in the entity edit form.
  $form['sid'] = array(
    '#type' => 'value',
    '#value' => $section->sid,
  );

  // Get section uri.
  $uri = section_uri($section);

  return confirm_form($form,
    t('Are you sure you want to delete %label?', array('%label' => $section->label)),
    $uri['path'],
    t('This action cannot be undone.'),
    t('Delete'),
    t('Cancel')
  );
}

/**
 * Execute section deletion
 */
function section_delete_confirm_submit($form, &$form_state) {
  if ($form_state['values']['confirm']) {
    $section = section_load($form_state['values']['sid']);
    section_delete($form_state['values']['sid']);
    watchdog('content', '@bundle: deleted %label.', array('@bundle' => section_bundle_get_name($section), '%label' => $section->label));
    drupal_set_message(t('@bundle %label has been deleted.', array('@bundle' => section_bundle_get_name($section), '%label' => $section->label)));
  }

  $form_state['redirect'] = 'admin/section';
}
